#ifndef _BILLBOARDGROUP_H
#define _BILLBOARDGROUP_H

#include <BBCPrerequisites.h>

namespace BBC {

// Forward declarations
class BillboardGroup;


namespace boost
{
    void intrusive_ptr_add_ref(BillboardGroup * p);
    void intrusive_ptr_release(BillboardGroup * p);
};

class BillboardGroup 
{
private:
    long references;
    friend void boost::intrusive_ptr_add_ref(BillboardGroup * p);
    friend void boost::intrusive_ptr_release(BillboardGroup * p);

  protected:
	std::vector<unsigned int> mBillboardHandleList;

  public:

    BillboardGroup();

	virtual ~BillboardGroup();

    unsigned int getBillboardHandle(unsigned int iBillboard);

    void addBillboardHandle(unsigned int iBillboardHandle);

	unsigned int getNumBillboards();

};

// class specific addref/release implementation
// the two function overloads must be in the boost namespace on most compilers:
namespace boost
{
 inline void intrusive_ptr_add_ref(BillboardGroup * p)
  {
    // increment reference count of object *p
    ++(p->references);
  }



 inline void intrusive_ptr_release(BillboardGroup * p)
  {
   // decrement reference count, and delete object when reference count reaches 0
   if (--(p->references) == 0)
     delete p;
  } 
} // namespace boost

typedef ::boost::intrusive_ptr<BillboardGroup> BillboardGroupPtr;

}
#endif
